/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Manuel Meier
 *   Co-authors:
 *      ...
 *
 */

/**
    @file OrxoKartOrigin.cc
    @brief The OrxoKartOrigin is a StaticEntity which represents the level of the minigame. All platforms, enemies and items are attached to the OrxoKart.
*/

#include "OrxoKartOrigin.h"
#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "OrxoKart.h"

namespace orxonox
{
    RegisterClass(OrxoKartOrigin);

    OrxoKartOrigin::OrxoKartOrigin(Context* context) : StaticEntity(context)
    {
        RegisterObject(OrxoKartOrigin);
        checkGametype();
    }

    void OrxoKartOrigin::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(OrxoKartOrigin, XMLPort, xmlelement, mode);

        XMLPortParam(OrxoKartOrigin, "numCells", setNumCells, getNumCells, xmlelement, mode);
        XMLPortParam(OrxoKartOrigin, "cellSize", setCellSize, getCellSize, xmlelement, mode);
        XMLPortParam(OrxoKartOrigin, "level", setLevel, getLevel, xmlelement, mode);
    }

    void OrxoKartOrigin::checkGametype()
    {
        if (getGametype() != nullptr && this->getGametype()->isA(Class(OrxoKart)))
        {
            OrxoKart* kartGametype = orxonox_cast<OrxoKart*>(this->getGametype());
            kartGametype->setOrigin(this);
        }
    }
}
