/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Julien Kindle
 *   Co-authors:
 *      
 *
 */

/**
    @file SOBMushroom.cc
    @brief This is the class for the powerup mushroom
*/

#include "SOBMushroom.h"

#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "SOBFigure.h"
#include "util/Output.h"


namespace orxonox
{
    RegisterClass(SOBMushroom);

    SOBMushroom::SOBMushroom(Context* context) : MovableEntity(context)
    {
        RegisterObject(SOBMushroom);

        setAngularFactor(0.0);
        this->enableCollisionCallback();
        gravityAcceleration_ = 350.0;
        speed_ = 0.0;
        hasCollided_=false;
        goesRight_ = true;
        changeAllowed_ = true;
        changedOn_ = 0.0;
       
    }

    

    void SOBMushroom::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(SOBMushroom, XMLPort, xmlelement, mode);
        XMLPortParam(SOBMushroom, "speed", setSpeed, getSpeed, xmlelement, mode);


     }

   
    bool SOBMushroom::collidesAgainst(WorldEntity* otherObject, const btCollisionShape* ownCollisionShape, btManifoldPoint& contactPoint) {


        if (changeAllowed_ && otherObject->getMass() != -1) {
                goesRight_ = !goesRight_;
                changeAllowed_ = false;
        }

        return true;
    }


    void SOBMushroom::tick(float dt)
    {
        SUPER(SOBMushroom, tick, dt);

        //Exactly like the Gumba class, if collided, turn direction of Mushroom and wait 200ms so we don't get two collisions from the same wall
        if (!changeAllowed_) {
            changedOn_+= dt;
            if (changedOn_> 0.200) {
                changeAllowed_ = true;
                changedOn_ = 0.0;

            }
        }
        

        int dir = 1;
        if (!goesRight_)
            dir = -1;

        Vector3 velocity = getVelocity();
        velocity.z -= gravityAcceleration_*dt;
        velocity.x = dir*speed_;
        setVelocity(velocity);
    }


}
