/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "WagnisProjectile.h"

#include "core/config/ConfigValueIncludes.h"
#include "core/CoreIncludes.h"
#include "core/GameMode.h"
#include "core/command/Executor.h"

#include "worldentities/pawns/Pawn.h"

namespace orxonox
{
    RegisterClass(WagnisProjectile);

    WagnisProjectile::WagnisProjectile(Context* context) : MovableEntity(context), BasicProjectile()
    {
        RegisterObject(WagnisProjectile);

        this->setConfigValues();

        // Get notification about collisions
        if (GameMode::isMaster())
        {
            this->setMass(0.1f);
            this->enableCollisionCallback();
            this->setCollisionResponse(false);
            this->setCollisionType(CollisionType::Dynamic);

            // Create a sphere collision shape and attach it to the projectile.
            collisionShape_ = new SphereCollisionShape(this->getContext());
            setCollisionShapeRadius(3.0f);
            this->attachCollisionShape(collisionShape_);

            this->destroyTimer_.setTimer(this->lifetime_, false, createExecutor(createFunctor(&BasicProjectile::destroyObject, this)));
        }
    }

    WagnisProjectile::~WagnisProjectile()
    {
    }

    void WagnisProjectile::setConfigValues()
    {
        SetConfigValue(lifetime_, 4.0f).description("The time in seconds a projectile stays alive");
    }

    void WagnisProjectile::tick(float dt)
    {
        SUPER(WagnisProjectile, tick, dt);

        if (!this->isActive())
            return;

        this->destroyCheck();
    }

    bool WagnisProjectile::collidesAgainst(WorldEntity* otherObject, const btCollisionShape* cs, btManifoldPoint& contactPoint)
    {
        return this->processCollision(otherObject, contactPoint, cs);
    }

    void WagnisProjectile::setCollisionShapeRadius(float radius)
    {
        if (collisionShape_ != nullptr && radius > 0)
        {
            collisionShape_->setRadius(radius);
        }        
    }
}
