/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Leon Rigoni
 *
 */
#ifndef _ShootableObstacle_H__
#define _ShootableObstacle_H__
#include "worldentities/pawns/Pawn.h"

namespace orxonox // tolua_export
{
    /**
    @brief
        This class encapsulates a pawn (so an object with health) with the addition, that it damages other objects when it hits them.
    */

    class _OrxonoxExport ShootableObstacle
        : public Pawn
    {
        public:
            ShootableObstacle(Context* context);
            virtual void XMLPort(Element& xmlelement, XMLPort::Mode mode) override;
            virtual bool collidesAgainst(WorldEntity* otherObject, const btCollisionShape* cs, btManifoldPoint& contactPoint) override;

            inline void setCollisionDamage(float c)
                    { this->collisionDamage_ = c; }

            inline float getCollisionDamage()
                { return this->collisionDamage_; }

            inline void setEnableCollisionDamage(bool c)
            {
                this->enableCollisionDamage_ = c;
                this->enableCollisionCallback();
            }

            inline bool getEnableCollisionDamage()
                { return this->enableCollisionDamage_; }

        private:
            float collisionDamage_;
            bool enableCollisionDamage_;
    };
}

#endif
